"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
exports.floor = exports.ceil = exports.exponentiation = exports.sqrt = exports.div = exports.mul = exports.sub = exports.add = exports.abs = exports.neg = void 0;
var jsbi_1 = __importDefault(require("jsbi"));
var constants_js_1 = require("./constants.js");
var constructors_js_1 = require("./constructors.js");
var predicates_js_1 = require("./predicates.js");
var relational_js_1 = require("./relational.js");
function neg(a) {
    return constructors_js_1.make_big_float(jsbi_1["default"].unaryMinus(a.coefficient), a.exponent);
}
exports.neg = neg;
function abs(a) {
    return predicates_js_1.is_negative(a) ? neg(a) : a;
}
exports.abs = abs;
function conform_op(op) {
    return function (a, b) {
        var differential = a.exponent - b.exponent;
        return differential === 0
            ? constructors_js_1.make_big_float(op(a.coefficient, b.coefficient), a.exponent)
            : differential > 0
                ? constructors_js_1.make_big_float(op(jsbi_1["default"].multiply(a.coefficient, jsbi_1["default"].exponentiate(constants_js_1.BIGINT_TEN, jsbi_1["default"].BigInt(differential))), b.coefficient), b.exponent)
                : constructors_js_1.make_big_float(op(a.coefficient, jsbi_1["default"].multiply(b.coefficient, jsbi_1["default"].exponentiate(constants_js_1.BIGINT_TEN, jsbi_1["default"].BigInt(-differential)))), a.exponent);
    };
}
exports.add = conform_op(jsbi_1["default"].add);
exports.sub = conform_op(jsbi_1["default"].subtract);
function mul(multiplicand, multiplier) {
    return constructors_js_1.make_big_float(jsbi_1["default"].multiply(multiplicand.coefficient, multiplier.coefficient), multiplicand.exponent + multiplier.exponent);
}
exports.mul = mul;
function div(dividend, divisor, precision) {
    if (precision === void 0) { precision = constants_js_1.PRECISION; }
    if (predicates_js_1.is_zero(dividend) || predicates_js_1.is_zero(divisor)) {
        return constants_js_1.ZERO;
    }
    var coefficient = dividend.coefficient, exponent = dividend.exponent;
    exponent -= divisor.exponent;
    if (typeof precision !== "number") {
        precision = constructors_js_1.number(precision);
    }
    if (exponent > precision) {
        coefficient = jsbi_1["default"].multiply(coefficient, jsbi_1["default"].exponentiate(constants_js_1.BIGINT_TEN, jsbi_1["default"].BigInt(exponent - precision)));
        exponent = precision;
    }
    coefficient = jsbi_1["default"].divide(coefficient, divisor.coefficient);
    return constructors_js_1.make_big_float(coefficient, exponent);
}
exports.div = div;
function sqrt(n) {
    var x = n;
    var y = constants_js_1.ONE;
    while (relational_js_1.gt(exports.sub(x, y), constants_js_1.EPSILON)) {
        x = div(exports.add(x, y), constants_js_1.TWO);
        y = div(n, x);
    }
    return x;
}
exports.sqrt = sqrt;
function exponentiation(base, exp) {
    if (relational_js_1.eq(exp, constants_js_1.ZERO)) {
        return constants_js_1.ONE;
    }
    if (predicates_js_1.is_negative(exp)) {
        return div(constants_js_1.ONE, exponentiation(base, abs(exp)));
    }
    if (exp.exponent === 0) {
        var result = base;
        var n = 1;
        while (n !== constructors_js_1.number(exp)) {
            result = mul(result, base);
            n += 1;
        }
        return result;
    }
    if (relational_js_1.gt(exp, constants_js_1.ONE) || relational_js_1.eq(exp, constants_js_1.ONE)) {
        var temp = exponentiation(base, div(exp, constants_js_1.TWO));
        return mul(temp, temp);
    }
    var low = constants_js_1.ZERO;
    var high = constants_js_1.ONE;
    var sqr = sqrt(base);
    var acc = sqr;
    var mid = div(high, constants_js_1.TWO);
    while (relational_js_1.gt(abs(exports.sub(mid, exp)), constants_js_1.EPSILON)) {
        sqr = sqrt(sqr);
        if (relational_js_1.lt(mid, exp) || relational_js_1.eq(mid, exp)) {
            low = mid;
            acc = mul(acc, sqr);
        }
        else {
            high = mid;
            acc = mul(acc, div(constants_js_1.ONE, sqr));
        }
        mid = div(exports.add(low, high), constants_js_1.TWO);
    }
    return acc;
}
exports.exponentiation = exponentiation;
function ceil(n) {
    if (predicates_js_1.is_integer(n)) {
        return n;
    }
    else {
        return constructors_js_1.make_big_float(jsbi_1["default"].add(constructors_js_1.integer(n).coefficient, constants_js_1.BIGINT_ONE), 0);
    }
}
exports.ceil = ceil;
function floor(n) {
    return constructors_js_1.integer(n);
}
exports.floor = floor;
