"use strict";
exports.__esModule = true;
exports.BigFloat = void 0;
var arithmetic_js_1 = require("./arithmetic.js");
var constructors_js_1 = require("./constructors.js");
var predicates_js_1 = require("./predicates.js");
var relational_js_1 = require("./relational.js");
var BigFloat = /** @class */ (function () {
    function BigFloat(n) {
        this.dp = this.decimalPlaces;
        this.isNeg = this.isNegative;
        this.eq = this.equals;
        this.gt = this.greaterThan;
        this.gte = this.greaterThanOrEqualTo;
        this.lt = this.lessThan;
        this.lte = this.lessThanOrEqualTo;
        this.isInt = this.isInteger;
        this.isPos = this.isPositive;
        this.abs = this.absoluteValue;
        this.neg = this.negated;
        this.sqrt = this.squareRoot;
        this.div = this.dividedBy;
        this.sub = this.minus;
        this.add = this.plus;
        this.mul = this.times;
        this.pow = this.toPower;
        var _a = constructors_js_1.make(n), exponent = _a.exponent, coefficient = _a.coefficient;
        this.exponent = exponent;
        this.coefficient = coefficient;
    }
    BigFloat.prototype.decimalPlaces = function () { return -this.exponent; };
    BigFloat.prototype.isInteger = function () { return predicates_js_1.is_integer(this); };
    BigFloat.prototype.isNegative = function () { return predicates_js_1.is_negative(this); };
    BigFloat.prototype.isPositive = function () { return predicates_js_1.is_positive(this); };
    BigFloat.prototype.isZero = function () { return predicates_js_1.is_zero(this); };
    BigFloat.prototype.toString = function () { return constructors_js_1.string(this); };
    BigFloat.prototype.equals = function (y) { return relational_js_1.eq(this, constructors_js_1.make(y)); };
    BigFloat.prototype.greaterThan = function (y) { return relational_js_1.gt(this, constructors_js_1.make(y)); };
    BigFloat.prototype.greaterThanOrEqualTo = function (y) { return relational_js_1.gte(this, constructors_js_1.make(y)); };
    BigFloat.prototype.lessThan = function (y) { return relational_js_1.lt(this, constructors_js_1.make(y)); };
    BigFloat.prototype.lessThanOrEqualTo = function (y) { return relational_js_1.lte(this, constructors_js_1.make(y)); };
    BigFloat.prototype.absoluteValue = function () { return new BigFloat(arithmetic_js_1.abs(this)); };
    BigFloat.prototype.negated = function () { return new BigFloat(arithmetic_js_1.neg(this)); };
    BigFloat.prototype.squareRoot = function () { return new BigFloat(arithmetic_js_1.sqrt(this)); };
    BigFloat.prototype.dividedBy = function (y) { return new BigFloat(arithmetic_js_1.div(this, constructors_js_1.make(y))); };
    BigFloat.prototype.minus = function (y) { return new BigFloat(arithmetic_js_1.sub(this, constructors_js_1.make(y))); };
    BigFloat.prototype.plus = function (y) { return new BigFloat(arithmetic_js_1.add(this, constructors_js_1.make(y))); };
    BigFloat.prototype.times = function (y) { return new BigFloat(arithmetic_js_1.mul(this, constructors_js_1.make(y))); };
    BigFloat.prototype.toPower = function (y) { return new BigFloat(arithmetic_js_1.exponentiation(this, constructors_js_1.make(y))); };
    BigFloat.prototype.ceil = function () { return new BigFloat(arithmetic_js_1.ceil(this)); };
    BigFloat.prototype.floor = function () { return new BigFloat(arithmetic_js_1.floor(this)); };
    return BigFloat;
}());
exports.BigFloat = BigFloat;
